@@ -1,11 +1,16 @@
 # -*- coding: utf-8 -*-
 
+from itertools import chain
+from random import choice
+from random import sample
+from random import seed
+from time import time
+
+import logging
 import os
 import string
-from itertools import chain
-from time import time
-from random import seed, choice, sample
-import logging
+
+
 logger = logging.getLogger("imio.helpers")
 
 
@@ -13,11 +18,11 @@ def is_develop_environment():
     """
         Test if the environment variable named IS_DEV_ENV is added by the buildout and get the value
     """
-    logger.info('IS_DEV_ENV is deprecated, please use ENV variable.')
     TRUISMS = ['yes', 'y', 'true', 'on']
     var = os.getenv('IS_DEV_ENV', 'false')
 
     if var.lower() in TRUISMS:
+        logger.info('IS_DEV_ENV is deprecated, please use ENV variable.')
         return True
     elif get_environment() == 'dev':
         return True
