@@ -12,10 +12,21 @@ from plone.app.testing import TEST_USER_NAME
 from plone.testing import z2
 
 import imio.helpers
-
+import logging
+import sys
 import unittest
 
 
+def testing_logger(logger_name=''):
+    logger = logging.getLogger(logger_name)
+    logger.setLevel(logging.DEBUG)
+    handler = logging.StreamHandler(sys.stdout)
+    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
+    handler.setFormatter(formatter)
+    logger.addHandler(handler)
+    return logger
+
+
 class PloneWithHelpersLayer(PloneSandboxLayer):
 
     defaultBases = (PLONE_FIXTURE,)
@@ -33,11 +44,15 @@ class PloneWithHelpersLayer(PloneSandboxLayer):
         # Install into Plone site using portal_setup
         applyProfile(portal, 'imio.helpers:testing')
 
+        # use intranet_workflow for every types
+        portal.portal_workflow.setDefaultChain('intranet_workflow')
         # Login and create some test content
         setRoles(portal, TEST_USER_ID, ['Manager'])
         login(portal, TEST_USER_NAME)
         folder_id = portal.invokeFactory('Folder', 'folder')
         portal[folder_id].reindexObject()
+        folder2_id = portal.invokeFactory('Folder', 'folder2')
+        portal[folder2_id].reindexObject()
 
         # Commit so that the test browser sees these objects
         import transaction
