@@ -1,14 +1,15 @@
 # -*- coding: utf-8 -*-
 
-from plone import api
-
-from imio.helpers.testing import IntegrationTestCase
-from imio.helpers.catalog import addOrUpdateIndexes
+from collections import OrderedDict
 from imio.helpers.catalog import addOrUpdateColumns
+from imio.helpers.catalog import addOrUpdateIndexes
 from imio.helpers.catalog import get_intid
-from imio.helpers.catalog import removeIndexes
+from imio.helpers.catalog import reindexIndexes
 from imio.helpers.catalog import removeColumns
+from imio.helpers.catalog import removeIndexes
 from imio.helpers.catalog import ZCTextIndexInfo
+from imio.helpers.testing import IntegrationTestCase
+from plone import api
 from plone.app.testing import applyProfile
 from zope.component import getUtility
 from zope.intid.interfaces import IIntIds
@@ -37,6 +38,32 @@ class TestCatalogModule(IntegrationTestCase):
         self.assertTrue(len(brains) == 1)
         self.assertTrue(brains[0].getObject().UID() == priorDocument.UID())
 
+    def test_addOrUpdateSeveralIndexes(self):
+        """
+        It is possible to pass several new indexes to addOrUpdateIndexes,
+        in this case, only new indexes are added and updated.
+        """
+        # for now reversedUID index does not exist...
+        self.assertTrue('reversedUID' not in self.catalog.indexes())
+        # and add an existing index UID
+        self.assertTrue(self.catalog.Indexes['UID'].getTagName() == 'UUIDIndex')
+        # add a document prior to adding the index
+        priorDocument = api.content.create(type='Document',
+                                           id='prior-document',
+                                           container=self.portal)
+        addOrUpdateIndexes(
+            self.portal,
+            # use an ordered dict so UID, an existing index, is after the new one
+            OrderedDict([
+                ('reversedUID', ('FieldIndex', {})),
+                ('UID', ('UUIDIndex', {}))])
+        )
+        self.assertTrue('reversedUID' in self.portal.portal_catalog.indexes())
+        # moreover existing objects were updated
+        brains = self.catalog(reversedUID=priorDocument.UID()[::-1])
+        self.assertTrue(len(brains) == 1)
+        self.assertTrue(brains[0].getObject().UID() == priorDocument.UID())
+
     def test_updateIndexType(self):
         """
         If an index already exists with a given type, it can be updated
@@ -175,3 +202,13 @@ class TestCatalogModule(IntegrationTestCase):
         new_id = get_intid(obj, intids=intids)
         self.assertNotEqual(new_id, obj_id)
         self.assertNotEqual(new_id, 1)
+
+    def test_reindexIndexes(self):
+        self.assertFalse(self.portal.portal_catalog(Title='specific'))
+        obj = api.content.create(container=self.portal.folder, id='tt', type='testingtype')
+        obj.title = 'specific'
+        self.assertFalse(self.portal.portal_catalog(Title='specific'))
+        reindexIndexes(self.portal, ['Title'])
+        res = self.portal.portal_catalog(Title='specific')
+        self.assertEqual(len(res), 1)
+        self.assertEqual(res[0].UID, obj.UID())
