@@ -1,16 +1,31 @@
 # -*- coding: utf-8 -*-
+from Acquisition import aq_base
+from os import path
+from plone import api
+from plone.app.imaging.scale import ImageScale
+from plone.outputfilters.browser.resolveuid import uuidToURL
+from plone.outputfilters.filters.resolveuid_and_caption import ResolveUIDAndCaptionFilter
+from Products.CMFPlone.utils import safe_unicode
+from zExceptions import NotFound
+from zope.container.interfaces import INameChooser
+
 import cgi
+import logging
 import lxml.html
 import os
+import pkg_resources
 import types
 import urllib
-from os import path
-from Acquisition import aq_base
-from zope.container.interfaces import INameChooser
-from plone import api
-from Products.CMFPlone.utils import safe_unicode
-from plone.app.imaging.scale import ImageScale
-from plone.outputfilters.filters.resolveuid_and_caption import ResolveUIDAndCaptionFilter
+
+
+logger = logging.getLogger('imio.helpers:xhtml')
+
+try:
+    HAS_CKEDITOR = True
+    pkg_resources.get_distribution('collective.ckeditor')
+except pkg_resources.DistributionNotFound:
+    HAS_CKEDITOR = False
+
 
 CLASS_TO_LAST_CHILDREN_NUMBER_OF_CHARS_DEFAULT = 240
 # Xhtml tags that may contain content
@@ -61,13 +76,13 @@ def removeBlanks(xhtmlContent, pretty_print=False):
 
     for el in tree.getchildren():
         # el can be a subtree, like <ul><li>...</li></ul> we must consider entire rendering of it
-        if xhtmlContentIsEmpty(el):
+        if xhtmlContentIsEmpty(el, tagWithAttributeIsNotEmpty=False):
             el.getparent().remove(el)
     # only return children of the <special_tag>
     return ''.join([lxml.html.tostring(x,
-                                       encoding='utf-8',
+                                       encoding='ascii',
                                        pretty_print=pretty_print,
-                                       method='xml')
+                                       method='html')
                     for x in tree.iterchildren()])
 
 
@@ -107,7 +122,7 @@ def addClassToContent(xhtmlContent, css_class, pretty_print=False):
     res = ''.join([lxml.html.tostring(x,
                                       encoding='ascii',
                                       pretty_print=pretty_print,
-                                      method='xml') for x in children])
+                                      method='html') for x in children])
     return res
 
 
@@ -177,7 +192,7 @@ def addClassToLastChildren(xhtmlContent,
     res = ''.join([lxml.html.tostring(x,
                                       encoding='ascii',
                                       pretty_print=pretty_print,
-                                      method='xml') for x in tree.iterchildren()])
+                                      method='html') for x in tree.iterchildren()])
     return res
 
 
@@ -214,7 +229,7 @@ def markEmptyTags(xhtmlContent,
     return ''.join([lxml.html.tostring(x,
                                        encoding='utf-8',
                                        pretty_print=pretty_print,
-                                       method='xml')
+                                       method='html')
                     for x in tree.iterchildren()])
 
 
@@ -235,7 +250,7 @@ def removeCssClasses(xhtmlContent,
     return ''.join([lxml.html.tostring(x,
                                        encoding='utf-8',
                                        pretty_print=pretty_print,
-                                       method='xml')
+                                       method='html')
                     for x in tree.iterchildren()])
 
 
@@ -270,8 +285,9 @@ def imagesToPath(context, xhtmlContent, pretty_print=False):
     portal_url = portal.absolute_url()
     for img in imgs:
         # check if it is a local or an external image
-        img_src = img.attrib['src']
-        if img_src.startswith('http') and not img_src.startswith(portal_url):
+        img_src = img.attrib.get('src', None)
+        # wrong <img> without src or external image
+        if not img_src or (img_src.startswith('http') and not img_src.startswith(portal_url)):
             continue
         # here, we have an image contained in the portal
         # either absolute path (http://...) or relative (../images/myimage.png)
@@ -282,13 +298,14 @@ def imagesToPath(context, xhtmlContent, pretty_print=False):
             try:
                 # get the image but remove leading '/'
                 imageObj = portal.unrestrictedTraverse(img_src[1:])
-            except (KeyError, AttributeError):
+            except (KeyError, AttributeError, NotFound):
                 continue
         # relative path
         else:
             try:
                 imageObj = context.unrestrictedTraverse(img_src)
-            except (KeyError, AttributeError):
+            # in case we have a wrong resolveuid/unknown_uid, it raises NotFound
+            except (KeyError, AttributeError, NotFound):
                 continue
         # maybe we have a ImageScale instead of the real Image object?
         if isinstance(imageObj, ImageScale):
@@ -305,7 +322,7 @@ def imagesToPath(context, xhtmlContent, pretty_print=False):
     return ''.join([lxml.html.tostring(x,
                                        encoding='ascii',
                                        pretty_print=pretty_print,
-                                       method='xml') for x in tree.iterchildren()])
+                                       method='html') for x in tree.iterchildren()])
 
 
 def storeImagesLocally(context,
@@ -313,7 +330,8 @@ def storeImagesLocally(context,
                        imagePortalType='Image',
                        store_external_images=True,
                        store_internal_images=True,
-                       pretty_print=False):
+                       pretty_print=False,
+                       force_resolve_uid=False):
     """If images are found in the given p_xhtmlContent,
        we download it and stored it in p_context, this way we ensure that it will
        always be available in case the external/internal image image disappear.
@@ -325,14 +343,25 @@ def storeImagesLocally(context,
         """ """
         # get image from URL
         img_path = img_src.replace(portal_url, '')
-        img_path = img_path.lstrip('/')
         img_path = path.join(portal.absolute_url_path(), img_path)
+        # path can not start with a '/'
+        img_path = img_path.lstrip('/')
+
         # right, traverse to image
         try:
             imageObj = portal.unrestrictedTraverse(img_path)
-        except KeyError:
+        except (KeyError, AttributeError):
             # wrong img_path
+            logger.warning(
+                'In \'storeImagesLocally\', could not traverse '
+                'img_path \'{0}\' for \'{1}\'!'.format(
+                    img_path, context.absolute_url()))
             return None, None
+
+        # not an image
+        if getattr(imageObj, 'portal_type', None) != 'Image':
+            return None, None
+
         filename = imageObj.getId()
         data = imageObj.getBlobWrapper().data
         return filename, data
@@ -345,6 +374,8 @@ def storeImagesLocally(context,
         except IOError:
             # url not existing
             return None, None
+
+        # not an image
         if not downloaded_img_infos.maintype == 'image':
             return None, None
 
@@ -375,7 +406,9 @@ def storeImagesLocally(context,
         return xhtmlContent
     portal = api.portal.get()
     portal_url = portal.absolute_url()
-    context_url = context.absolute_url()
+    # make sure context_url ends with a '/' to avoid mismatch if folder holding the
+    # image absolute_url starts with context_url
+    context_url = context.absolute_url() + '/'
     # adapter to generate a valid id, it needs a container, so it will be context or it's parent
     try:
         name_chooser = INameChooser(context)
@@ -385,16 +418,34 @@ def storeImagesLocally(context,
     # return received xhtmlContent if nothing was changed
     changed = False
     for img in imgs:
-        img_src = img.attrib['src']
+        original_img_src = img.get('src', '')
+
         # we only handle http stored images
-        if not img_src.startswith('http'):
+        if not original_img_src.startswith('http') and 'resolveuid' not in original_img_src:
             continue
         filename = data = None
         # external images
-        if store_external_images and not img_src.startswith(portal_url):
-            filename, data = _handle_external_image(img_src)
+        if store_external_images and not original_img_src.startswith(portal_url) and \
+           'resolveuid' not in original_img_src:
+            filename, data = _handle_external_image(original_img_src)
+
+        # image in portal but not already stored in context
+        # handle images using resolveuid
+        end_of_url = None
+        img_src = original_img_src
+        if 'resolveuid' in original_img_src:
+            # manage cases like :
+            # - resolveuid/2ff7ea3317df43438a09edfa84965b13
+            # - resolveuid/2ff7ea3317df43438a09edfa84965b13/image_preview
+            # - http://portal_url/resolveuid/2ff7ea3317df43438a09edfa84965b13
+            img_uid = original_img_src.split('resolveuid/')[-1].split('/')[0]
+            # save end of url if any, like /image_preview
+            end_of_url = original_img_src.split(img_uid)[-1]
+            if end_of_url == img_uid:
+                end_of_url = None
+            # if not found, like when copy/paste HTML code containing resolveuid, use '' instead None
+            img_src = uuidToURL(img_uid) or ''
 
-        # image in portal but not already stored incontext
         if store_internal_images and \
            img_src.startswith(portal_url) and \
            not img_src.startswith(context_url):
@@ -408,7 +459,19 @@ def storeImagesLocally(context,
         name = name_chooser.chooseName(filename, context)
         new_img_id = context.invokeFactory(imagePortalType, id=name, title=name, file=data)
         new_img = getattr(context, new_img_id)
-        img.attrib['src'] = new_img.absolute_url()
+        # store a resolveuid if using it, the absolute_url to image if not
+        if force_resolve_uid or \
+           original_img_src.startswith('resolveuid') or \
+           (HAS_CKEDITOR and
+                hasattr(portal.portal_properties, 'ckeditor_properties') and
+                portal.portal_properties.ckeditor_properties.allow_link_byuid):
+            new_img_src = 'resolveuid/{0}'.format(new_img.UID())
+        else:
+            new_img_src = new_img.absolute_url()
+        if end_of_url:
+            new_img_src = '/'.join([new_img_src, end_of_url.strip('/')])
+
+        img.attrib['src'] = new_img_src
 
     if not changed:
         return xhtmlContent
@@ -416,4 +479,16 @@ def storeImagesLocally(context,
     return ''.join([lxml.html.tostring(x,
                                        encoding='utf-8',
                                        pretty_print=pretty_print,
-                                       method='xml') for x in tree.iterchildren()])
+                                       method='html') for x in tree.iterchildren()])
+
+
+def object_link(obj, view='view', attribute='Title', content=''):
+    """ Returns an html link for the given object """
+    href = view and "%s/%s" % (obj.absolute_url(), view) or obj.absolute_url()
+    if not content:
+        if not hasattr(obj, attribute):
+            attribute = 'Title'
+        content = getattr(obj, attribute)
+        if callable(content):
+            content = content()
+    return u'<a href="%s">%s</a>' % (href, safe_unicode(content))
